import { AdminConsoleKey, I18nKey } from '@logto/phrases';
import { AppearanceMode, Application } from '@logto/schemas';
import { useNavigate } from 'react-router-dom';
import useSWR from 'swr';

import CheckDemoDark from '@/assets/images/check-demo-dark.svg';
import CheckDemo from '@/assets/images/check-demo.svg';
import CreateAppDark from '@/assets/images/create-app-dark.svg';
import CreateApp from '@/assets/images/create-app.svg';
import CustomizeDark from '@/assets/images/customize-dark.svg';
import Customize from '@/assets/images/customize.svg';
import FurtherReadingsDark from '@/assets/images/further-readings-dark.svg';
import FurtherReadings from '@/assets/images/further-readings.svg';
import OneClickDark from '@/assets/images/one-click-dark.svg';
import OneClick from '@/assets/images/one-click.svg';
import PasswordlessDark from '@/assets/images/passwordless-dark.svg';
import Passwordless from '@/assets/images/passwordless.svg';
import { RequestError } from '@/hooks/use-api';
import useSettings from '@/hooks/use-settings';
import { useTheme } from '@/hooks/use-theme';

type GetStartedMetadata = {
  id: string;
  title: AdminConsoleKey;
  subtitle: AdminConsoleKey;
  icon: SvgComponent;
  buttonText: I18nKey;
  isComplete?: boolean;
  isHidden?: boolean;
  onClick: () => void;
};

type Props = {
  checkDemoAppExists: boolean;
};

const useGetStartedMetadata = ({ checkDemoAppExists }: Props) => {
  const { settings, updateSettings } = useSettings();
  const theme = useTheme();
  const isLightMode = theme === AppearanceMode.LightMode;
  const { data: demoApp, error } = useSWR<Application, RequestError>(
    checkDemoAppExists && '/api/applications/demo_app',
    {
      shouldRetryOnError: (error: unknown) => {
        if (error instanceof RequestError) {
          return error.status !== 404;
        }

        return true;
      },
    }
  );
  const navigate = useNavigate();
  const isLoadingDemoApp = !demoApp && !error;
  const hideDemo = error?.status === 404;

  const data: GetStartedMetadata[] = [
    {
      id: 'checkDemo',
      title: 'get_started.card1_title',
      subtitle: 'get_started.card1_subtitle',
      icon: isLightMode ? CheckDemo : CheckDemoDark,
      buttonText: 'general.check_out',
      isComplete: settings?.checkDemo,
      isHidden: hideDemo,
      onClick: async () => {
        void updateSettings({ checkDemo: true });
        window.open('/demo-app', '_blank');
      },
    },
    {
      id: 'createApplication',
      title: 'get_started.card2_title',
      subtitle: 'get_started.card2_subtitle',
      icon: isLightMode ? CreateApp : CreateAppDark,
      buttonText: 'general.create',
      isComplete: settings?.createApplication,
      onClick: () => {
        navigate('/applications/create');
      },
    },
    {
      id: 'configurePasswordless',
      title: 'get_started.card3_title',
      subtitle: 'get_started.card3_subtitle',
      icon: isLightMode ? Passwordless : PasswordlessDark,
      buttonText: 'general.create',
      isComplete: settings?.configurePasswordless,
      onClick: () => {
        navigate('/connectors');
      },
    },
    {
      id: 'configureSocialSignIn',
      title: 'get_started.card4_title',
      subtitle: 'get_started.card4_subtitle',
      icon: isLightMode ? OneClick : OneClickDark,
      buttonText: 'general.set_up',
      onClick: () => {
        navigate('/connectors/social');
      },
    },
    {
      id: 'customizeSignInExperience',
      title: 'get_started.card5_title',
      subtitle: 'get_started.card5_subtitle',
      icon: isLightMode ? Customize : CustomizeDark,
      buttonText: 'general.customize',
      isComplete: settings?.customizeSignInExperience,
      onClick: () => {
        navigate('/sign-in-experience');
      },
    },
    {
      id: 'checkFurtherReadings',
      title: 'get_started.card6_title',
      subtitle: 'get_started.card6_subtitle',
      icon: isLightMode ? FurtherReadings : FurtherReadingsDark,
      buttonText: 'general.check_out',
      isComplete: settings?.checkFurtherReadings,
      onClick: () => {
        void updateSettings({ checkFurtherReadings: true });
        window.open('https://docs.logto.io/', '_blank');
      },
    },
  ];

  return {
    data,
    completedCount: data.filter(({ isComplete }) => isComplete).length,
    totalCount: data.length,
    isLoading: isLoadingDemoApp,
  };
};

export default useGetStartedMetadata;
